/*global TweenMax, TweenLite, Power4, Power3, $, loadImage
 */

import jquery from "jquery";
import loadImage from 'blueimp-load-image'

// polyfill for canvas.toBlob()
if (!HTMLCanvasElement.prototype.toBlob) {
    Object.defineProperty(HTMLCanvasElement.prototype, 'toBlob', {
        value: function (callback, type, quality) {

            const binStr = atob(this.toDataURL(type, quality).split(',')[1]),
                len = binStr.length,
                arr = new Uint8Array(len);

            for (let i = 0; i < len; i++) {
                arr[i] = binStr.charCodeAt(i);
            }

            callback(new Blob([arr], {type: type || 'image/png'}));
        }
    });
}

if (!Array.prototype.findIndex) {
    Object.defineProperty(Array.prototype, 'findIndex', {
        value: function (predicate) {
            // 1. Let O be ? ToObject(this value).
            if (this == null) {
                throw new TypeError('"this" is null or not defined');
            }

            const o = Object(this);

            // 2. Let len be ? ToLength(? Get(O, "length")).
            const len = o.length >>> 0;

            // 3. If IsCallable(predicate) is false, throw a TypeError exception.
            if (typeof predicate !== 'function') {
                throw new TypeError('predicate must be a function');
            }

            // 4. If thisArg was supplied, let T be thisArg; else let T be undefined.
            const thisArg = arguments[1];

            // 5. Let k be 0.
            let k = 0;

            // 6. Repeat, while k < len
            while (k < len) {
                // a. Let Pk be ! ToString(k).
                // b. Let kValue be ? Get(O, Pk).
                // c. Let testResult be ToBoolean(? Call(predicate, T, « kValue, k, O »)).
                // d. If testResult is true, return k.
                const kValue = o[k];
                if (predicate.call(thisArg, kValue, k, o)) {
                    return k;
                }
                // e. Increase k by 1.
                k++;
            }

            // 7. Return -1.
            return -1;
        }
    });
}


class NyrisIr {
    initSettings() {
        this.recognitionRequestId = 0;
        this.settings = {
            maxWidth: 512,
            maxHeight: 512,
            jpegQuality: 0.92,

            similaritySearch: true,
            similarityThreshold: null,
            regions: (location.search.indexOf("use.regions") >= 0),

            imageMatchingUrl: 'https://api.nyris.io/find/v1',
            imageMatchingSubmitManualUrl: 'https://api.nyris.io/find/v1/manual/',
            regionProposalUrl: 'https://api.nyris.io/find/v1/regions/',
            responseFormat: 'application/offers.nyris+json',

            debug: {
                classes: (location.search.indexOf("debug.classes") >= 0),
                resize: (location.search.indexOf("debug.resize") >= 0),
                crop: (location.search.indexOf("debug.crop") >= 0)
            }, // debug strings in query

            requestProducts(semanticOnly, blob, resolve, reject) {
                const headers = {
                    'X-Api-Key': this.settings.apiKey,
                    'Accept-Language': 'de,*;q=0.5',
                    'Accept': this.settings.responseFormat
                };
                if (semanticOnly)
                    headers['X-Only-Semantic-Search'] = 'yup';
                if (this.settings.similarityThreshold)
                    headers['X-Semantic-Threshold'] = this.settings.similarityThreshold;
                jquery.ajax({
                    url: this.settings.imageMatchingUrl,
                    headers: headers,
                    type: 'POST',
                    data: blob,
                    processData: false,
                    contentType: false,
                    success: (r) => resolve(
                        r.offerInfos.filter(offer => offer.img && offer.img.url)
                    ),
                    error: function (a, b, errorMsg) {
                        reject(errorMsg);
                    }
                });
            },
        };
    }

    constructor(settings) {
        this.initSettings();
        // merge settings with default settings
        Object.keys(settings).forEach(function (key) {
            this.settings[key] = settings[key];
        }.bind(this));
    }

    async requestProducts(semanticOnly, image) {
        const blob = await this.imageToBlob(this.scaleImage(image));
        return await new Promise((resolve, reject) => this.settings.requestProducts.bind(this)(semanticOnly, blob, resolve, reject));
    };

    async requestRegions(image) {
        const blob = await this.imageToBlob(this.scaleImage(image));
        this.recognitionRequestId++;
        let headers = {
            'X-Api-Key': this.settings.apiKey
        };
        return await jquery.ajax({
            url: this.settings.regionProposalUrl,
            headers: headers,
            type: 'POST',
            data: blob,
            processData: false,
            contentType: false
        }).promise();
    };

    async loadImage(dataUrlOrFile) {
        const options = {
            canvas: true,
            orientation: true
        };
        return await new Promise(resolve => loadImage(dataUrlOrFile, resolve, options));
    }

    scaleImage(imageOrCanvas, region = {}) {
        const options = {
            canvas: true,
            maxWidth: this.settings.maxWidth,
            maxHeight: this.settings.maxHeight,
            top: region.top,
            left: region.left,
            right: region.right,
            bottom: region.bottom
        };
        return loadImage.scale(imageOrCanvas, options);
    }

    async imageToBlob(image) {
        return await new Promise(resolve =>
            image.toBlob(resolve, 'image/jpeg', this.settings.jpegQuality)
        );
    };

    imageToDataUrl(image) {
        return image.toDataURL('image/jpeg', this.settings.jpegQuality);
    }
}


export default NyrisIr;