import jquery from "jquery";

import NyrisIr from "./nyris-ir.js";

import 'jquery-jcrop/css/jquery.Jcrop.min.css';
import template from '../templates/index.html';
import resultTemplate from '../templates/result.html';

import "jquery-jcrop";


class Nyris {
    constructor(config) {
        this.nyrisIm = new NyrisIr({apiKey: config.apiKey});
        this.config = config;
        this.dom = {};

        this.dom.mountPoint = jquery('#nyris-mount-point');
        this.dom.mountPoint.append(config.template);

        this.dom.icon = jquery('.nyris__icon');
        this.dom.window = jquery('.nyris__main');
        this.dom.close = jquery('.nyris__close');

        const screen = name => jquery(`.nyris__screen.nyris__${name}`);
        this.dom.allScreens = jquery('.nyris__screen');
        this.dom.screens = {
            hello: screen('hello'),
            wait: screen('wait'),
            singleResult: screen('success-single'),
            multipleResults: screen('success-multiple'),
            reselect: screen('reselect'),
            fail: screen('fail')
        };

        this.dom.hello = {
            dropZone: jquery('.nyris__hello-drop-zone'),
            imageInput: jquery('#nyris__hello-upload-input'),
            openCamera: jquery('#nyris__hello-open-camera')
        };

        this.dom.singleResult = {
            image: jquery('.nyris__success-single-image'),
            panel: jquery('.nyris__success-single-image-panel'),
            name: jquery('.nyris__success-single-panel-name'),
            price: jquery('.nyris__success-single-panel-description'),
            refine: jquery('.nyris__success-single-refine')
        };

        this.dom.multipleResults = {
            name: jquery('.nyris__success-multiple-name'),
            count: jquery('.nyris__success-multiple-count'),
            thumbnail: jquery('.nyris__success-multiple-thumb'),
            results: jquery('.nyris__success-multiple-result-list'),
            refine: jquery('.nyris__success-multiple-refine')
        };

        this.dom.reselect = {
            wrapper: this.dom.screens['reselect'].find('.nyris__main-content'),
            image: jquery('.nyris__reselect-image'),
            accept: jquery('.nyris__reselect-accept'),
            back: jquery('.nyris__reselect-back')
        };

        this.dom.fail = {
            restart: jquery('.nyris__fail-restart'),
            refine: jquery('.nyris__fail-reselect')
        };

        this.showScreen('hello');

        this.bindEvents();
    }

    closeAndReset() {
        this.showScreen('hello');
        this.dom.window.hide();
        this.dom.window.removeClass('nyris__main--wide');
    }

    bindEvents() {
        this.dom.icon.on('click', this.toggleNyris.bind(this));
        this.dom.close.on('click', this.closeAndReset.bind(this));

        this.dom.hello.imageInput.on('change', this.loadFileFromInput.bind(this));
        this.dom.hello.openCamera.on('change', this.loadFileFromInput.bind(this));
        this.dom.hello.dropZone.on('drop', this.handleFileDrop.bind(this));
        this.dom.hello.dropZone.on('dragover', Nyris.ignoreEvent);
        this.dom.hello.dropZone.on('dragenter', Nyris.ignoreEvent);
        this.dom.icon.on('drop', this.handleFileDropOntoIcon.bind(this));
        this.dom.icon.on('dragover', Nyris.ignoreEvent);
        this.dom.icon.on('dragenter', Nyris.ignoreEvent);
        this.dom.singleResult.refine.on('click', this.refineSelection.bind(this));
        this.dom.multipleResults.refine.on('click', this.refineSelection.bind(this));
        this.dom.fail.restart.on('click', this.restart.bind(this));
        this.dom.fail.refine.on('click', this.refineSelection.bind(this));
        this.dom.reselect.accept.on('click', this.acceptCrop.bind(this));
    }

    restart() {
        this.showScreen('hello');
        this.dom.window.removeClass('nyris__main--wide');
    }

    async acceptCrop() {
        this.croppedImageUrl = this.nyrisIm.imageToDataUrl(this.cropedImage);
        await this.startProcessing();
    }

    async showRefineSearch(regions) {
        if (this.jcrop) {
            this.jcrop.destroy();
            this.jcrop = null;
        }
        this.showScreen('reselect');
        if (regions.length === 0) {
            const tenPercentWidth = this.scaledImage.width/10;
            const tenPercentHeight = this.scaledImage.height/10;
            regions.push({
                region: {
                    left: tenPercentWidth,
                    top: tenPercentHeight,
                    right: this.scaledImage.width - tenPercentWidth,
                    bottom: this.scaledImage.height - tenPercentHeight
                }
            })
        }
        this.regions = regions
            .map((entry, idx) => Object.assign(entry.region, {
                index: idx,
                centerX: (entry.region.top + entry.region.bottom) / 2,
                centerY: (entry.region.left + entry.region.right) / 2,
                width: entry.region.right - entry.region.left,
                height: entry.region.bottom - entry.region.top,
                rightMargin: this.scaledImage.width - entry.region.right,
                bottomMargin: this.scaledImage.height - entry.region.bottom,
            }));

        const _this = this;
        const scaleFactor = this.originalImage.width / this.scaledImage.width;
        this.dom.reselect.image.Jcrop({
            boxWidth: this.dom.reselect.wrapper.width(),
            boxHeight: this.dom.reselect.wrapper.height(),
            trueSize: [this.scaledImage.width, this.scaledImage.height],
            onSelect: crop => {
                const region = {
                    left: crop.x * scaleFactor,
                    top: crop.y * scaleFactor,
                    right: (this.scaledImage.width - crop.x2) * scaleFactor,
                    bottom: (this.scaledImage.height - crop.y2) * scaleFactor
                };
                this.cropedImage = this.nyrisIm.scaleImage(this.originalImage, region);
            }
        }, function () {
            _this.jcrop = this;
        });

        await this.selectRegion(this.regions[0]);
    }


    async selectRegion(selectedRegion) {
        const holder = jquery('.jcrop-holder');
        const scaleFactor = Math.min(holder.width() / this.scaledImage.width, holder.height() / this.scaledImage.height);

        const coords = [
            selectedRegion.left,
            selectedRegion.top,
            selectedRegion.left + selectedRegion.width,
            selectedRegion.top + selectedRegion.height
        ];
        this.jcrop.setSelect(coords);

        const regionsWithBubbles = this.regions.filter((r) => r !== selectedRegion);
        holder
            .css('position', 'relative')
            .css('overflow', 'hidden')
            .find('.nyris__reselect-bubble').remove();
        regionsWithBubbles.map(region => {
            jquery(`<div class="nyris__reselect-bubble"><div class="nyris__reselect-bubble-inside"></div></div>`)
                .css('top', region.centerX * scaleFactor)
                .css('left', region.centerY * scaleFactor)
                .on('click', () => this.selectRegion(region))
                .appendTo(holder);
        });

    }

    async refineSelection() {
        this.showScreen('wait');
        this.dom.reselect.image
            .attr('src', this.originalImageUrl).attr('style', '');
        this.dom.window.removeClass('nyris__main--wide');
        try {
            this.scaledImage = this.nyrisIm.scaleImage(this.originalImage);
            const regions = await this.nyrisIm.requestRegions(this.scaledImage);
            await this.showRefineSearch(regions);
        } catch (err) {
            this.showScreen('fail', err);
        }
    }


    async handleFileDrop(event) {
        event.preventDefault();
        event.stopPropagation();

        const dataTransfer = event.originalEvent.dataTransfer;
        if (dataTransfer.items && dataTransfer.items.length > 0) {
            this.imageFile = dataTransfer.items[0].getAsFile();
        } else if (dataTransfer.files && dataTransfer.files.length > 0) {
            this.imageFile = dataTransfer.files[0];
        } else {
            return alert('The browser does not support file dragging or object dropped is not a file')
        }
        await this.loadImageFile(this.imageFile);
        await this.startProcessing();
    }

    async handleFileDropOntoIcon(event) {
        await this.handleFileDrop(event);
        this.dom.window.css('display', 'flex');
    }

    static ignoreEvent(event) {
        event.preventDefault();
        event.stopPropagation();
    }

    showScreen(name, msg) {
        this.dom.allScreens.hide();
        this.dom.screens[name].css('display', 'flex');
        if (name === "fail") console.log('showScreen', name, msg);
    }

    async loadFileFromInput(e) {
        this.imageFile = e.target.files[0];
        this.croppedImageUrl = null;
        this.cropedImage = null;
        this.scaledImage = null;
        await this.loadImageFile(this.imageFile);
        await this.startProcessing();
    }

    async loadImageFile(file) {
        this.originalImage = await this.nyrisIm.loadImage(file);
        this.originalImageUrl = this.nyrisIm.imageToDataUrl(this.originalImage);
    }

    toggleNyris() {
        if (this.dom.window.css('display') === 'none') {
            this.dom.window.css('display', 'flex');
        } else {
            this.dom.window.hide();
        }
    }

    renderResults(results) {
        const queryImageUrl = this.croppedImageUrl || this.originalImageUrl;
        if (results.length > 1) {
            this.dom.multipleResults.results
                .empty()
                .css('overflow-y', 'hidden')
                .scrollTop(0)
                .css('overflow-y', 'scroll');
            this.dom.multipleResults.count.text(results.length.toString());
            this.dom.multipleResults.thumbnail.css('background-image', `url(${queryImageUrl})`);
            results.map(offer => {
                    const t = jquery(resultTemplate);
                    t.find('.nyris__success-multiple-result-box').attr('href', offer.l);
                    t.find('.nyris__success-multiple-product-image').css('background-image', `url("${offer.img.url}")`);
                    t.find('.nyris__success-title').text(offer.title);
                    if (offer.p) t.find('.nyris__success-price').text(`${(offer.p.vi / 100).toFixed(2)} ${offer.p.c}`);
                    return t;
                }
            ).forEach(t => this.dom.multipleResults.results.append(t));
            this.dom.window.addClass('nyris__main--wide');
            this.showScreen('multipleResults');
            this.dom.multipleResults.results
                .css('overflow-y', 'hidden')
                .scrollTop(0)
                .css('overflow-y', 'scroll');
        } else if (results.length === 1) {
            const offer = results[0];
            this.dom.singleResult.name.text(offer.title);
            if (offer.p) this.dom.singleResult.price.text(`${(offer.p.vi / 100).toFixed(2)} ${offer.p.c}`);
            this.dom.singleResult.image.attr('href', offer.l);
            this.dom.singleResult.image.css('background-image', `url(${offer.img.url}`);
            this.showScreen('singleResult');
        } else {
            this.showScreen('fail', 'no results');
        }
    }

    async startProcessing() {
        let image = this.cropedImage || this.scaledImage || this.originalImage;
        this.showScreen('wait');
        try {
            const results = await this.nyrisIm.requestProducts(false, image);
            this.renderResults(results);
        } catch (e) {
            return this.showScreen('fail', e);
        }
    }
}

jquery(() => {
    jquery('<div id="nyris-mount-point"> </div>').prependTo('body');
    const nyris = new Nyris({
        template: template,
        apiKey: window.gNyrisApiKey
    })
});