const ExtractTextPlugin = require('extract-text-webpack-plugin');
const path = require('path');
const UglifyJsPlugin = require('uglifyjs-webpack-plugin')


module.exports = {
    entry: {
        nyris: ['./src/1.1/js/nyris.js', './src/1.1/styles/nyris.scss'],
    },
    output: {
        path: path.join(process.cwd(), 'publish/libs/js/1.1/search'),
        filename: "[name].min.js",
    },
    devtool: 'source-map',
    module: {
        rules: [{
            test: /\.s?css$/i,
            use: ExtractTextPlugin.extract([
                'css-loader',
                'postcss-loader',
                'sass-loader'
            ])
        },
            {
                test: /\.html$/i,
                use: 'raw-loader'
            },
            {
                test: /\.(jpe?g|png|gif|svg)$/i,
                loaders: [{
                    loader: 'url-loader',
                    options: {
                        limit: 8192
                    }
                }, {
                    loader: 'image-webpack-loader',
                    options: {
                        gifsicle: {
                            interlaced: false,
                        }
                    }
                }]
            },
            {
                test: /\.js$/i,
                exclude: /(node_modules|bower_components)/,
                use: {
                    loader: 'babel-loader',
                    options: {
                        presets: ['env'],
                        plugins: ['transform-runtime']
                    }
                }
            }
        ],
    },
    plugins: [
        new ExtractTextPlugin({
            filename: '[name].min.css',
        })
        , new UglifyJsPlugin({
            sourceMap: true
        })
    ]
}